//###########################################################################
//
// FILE:    Example_2834xADC_XINTF.c  
//
// TITLE:   Example ADC program for the TMS320C2834x Delfino DIM100 controlCARD
//
// ASSUMPTIONS:
//
//    This program requires the DSP2834x header files.
//
//    As supplied, this project is configured for "boot to SARAM" 
//    operation.  The 2834x Boot Mode table is shown below.  
//    For information on configuring the boot mode of an eZdsp, 
//    please refer to the documentation included with the eZdsp, 
//
//       $Boot_Table:
//
//         GPIO87   GPIO86     GPIO85   GPIO84
//          XA15     XA14       XA13     XA12
//           PU       PU         PU       PU
//        ==========================================
//            1        1          1        1    TI Test Only
//            1        1          1        0    SCI-A boot
//            1        1          0        1    SPI-A boot
//            1        1          0        0    I2C-A boot timing 1
//            1        0          1        1    eCAN-A boot timing 1
//            1        0          1        0    McBSP-A boot
//            1        0          0        1    Jump to XINTF x16
//            1        0          0        0    Jump to XINTF x32
//            0        1          1        1    eCAN-A boot timing 2
//            0        1          1        0    Parallel GPIO I/O boot
//            0        1          0        1    Parallel XINTF boot
//            0        1          0        0    Jump to SARAM	    <- "boot to SARAM"
//            0        0          1        1    Branch to check boot mode
//            0        0          1        0    I2C-A boot timing 2
//            0        0          0        1    Reserved
//            0        0          0        0    TI Test Only
//                                              Boot_Table_End$$
//
// DESCRIPTION:
//		
//		This example shows how to use the two ADS7865 ADCs on the 
//		TMS320C2834x Delfino DIM100 controlCARD. These external ADCs are
//		connected to the XINTF module of the C2834x device. This example
//		toggles GPIO 1 & 2 as input for the ADC to sample - the user should
//		connect these GPIOs to inputs A0 and B0 on the controlCARD docking
//		station.
//		
//		See the documentation in the doc folder for more information.
//
//###########################################################################
// $TI Release: 2834x Header Files V1.12 $
// $Release Date: March 2011 $
//###########################################################################

#include "DSP28x_Project.h"     // Device Headerfile and Examples Include File

// Prototype statements for functions found within this file.
interrupt void cpu_timer0_isr(void);
void GPIO_init(void);
void XINTF_init(void);
void ADC_init(void);
void EPWM_init(void);
void Run_ADC(void);

volatile Uint16* address_ptr;
int16 ADC1Data[256], ADC2Data[256], ndx;

#define CE0 1	//Use ADC 1 (connected to XINTF Zone 0)
#define CE7 1	//Use ADC 2 (connected to XINTF Zone 7)

#if CE0
	#define ADC1 (Uint16 volatile *)0x4000		//Zone 0 address for ADC1
#endif
#if CE7
	#define ADC2 (Uint16 volatile *)0x200000	//Zone 7 address for ADC2
#endif  

void main(void)
{

// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the DSP2834x_SysCtrl.c file.
   InitSysCtrl();	//SYSCLKOUT is set to 200MHz in this example

// Step 2. Initalize GPIO:
// This example function is found in the DSP2834x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
// InitGpio();  // Skipped for this example


// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
   DINT;

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the DSP2834x_PieCtrl.c file.
   InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
   IER = 0x0000;
   IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in DSP2834x_DefaultIsr.c.
// This function is found in DSP2834x_PieVect.c.
   InitPieVectTable();

// Interrupts that are used in this example are re-mapped to
// ISR functions found within this file.
   EALLOW;  // This is needed to write to EALLOW protected registers
   PieVectTable.TINT0 = &cpu_timer0_isr;
   EDIS;    // This is needed to disable write to EALLOW protected registers

// Step 4. Initialize the Device Peripheral. This function can be
//         found in DSP2834x_CpuTimers.c
   InitCpuTimers();   // For this example, only initialize the Cpu Timers

// Configure CPU-Timer 0 interrupt every 50 microseconds:
// 200MHz CPU Freq, 50 uS Period (in uSeconds)
   ConfigCpuTimer(&CpuTimer0, 200, 50);

// Configure GPIOs
   GPIO_init();

// Configure XINTF to talk to ADCs
   XINTF_init();

// Configure ADCs
   ADC_init();

// Configure PWM to trigger ADC2
   EPWM_init();

// To ensure precise timing, use write-only instructions to write to the entire register. Therefore, if any
// of the configuration bits are changed in ConfigCpuTimer and InitCpuTimers (in DSP2834x_CpuTimers.h), the
// below settings must also be updated.
   CpuTimer0Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0

// Enable CPU int1 which is connected to CPU-Timer 0
   IER |= M_INT1;

// Enable TINT0 in the PIE: Group 1 interrupt 7
   PieCtrlRegs.PIEIER1.bit.INTx7 = 1;

// Enable global Interrupts:
   EINT;   // Enable Global interrupt INTM

// Run ADC
   Run_ADC();

}  // end main

interrupt void cpu_timer0_isr(void)
{
	// CPU timer0 Interrupt Service Routine
	// Toggles GPIOs to create square waves as input for the ADC to sample
   
	CpuTimer0.InterruptCount++;
	GpioDataRegs.GPATOGGLE.bit.GPIO1 = 1; // Toggle GPIO1
	GpioDataRegs.GPATOGGLE.bit.GPIO2 = 1; // Toggle GPIO2
   
	// Acknowledge this interrupt to receive more interrupts from group 1
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}

void GPIO_init()
{
	// Configures the GPIOs for XINTF, ADCs, and GPIOs for ADC 
	// input and manual conversion start.
	
   	EALLOW;

	// Configure XINTF pins
   	GpioCtrlRegs.GPAMUX2.all = 0xAAAAAAAA;
   	GpioCtrlRegs.GPBMUX1.all = 0xFFFFFFC0;
   	GpioCtrlRegs.GPBMUX2.all = 0xAAAAAAAA; 
   	GpioCtrlRegs.GPCMUX1.all = 0xAAAAAAAA;
   	GpioCtrlRegs.GPCMUX2.all = 0x0000AAAA;
   	asm(" NOP");
   	asm(" NOP");

	// Disable pull-ups on XINTF Address, data and control pins.
	GpioCtrlRegs.GPAPUD.all |= 0xF0008FFF;
	GpioCtrlRegs.GPBPUD.all |= 0xFFFFFFF8;
	GpioCtrlRegs.GPCPUD.all |= 0x00FFFFFF; 
    asm(" NOP");
    asm(" NOP");

	// Make gpio 30,31 GPIO so as not to confuse XINTF Address
	GpioCtrlRegs.GPAMUX2.bit.GPIO30 = 0;	// 0=GPIO,  
	GpioCtrlRegs.GPADIR.bit.GPIO30 = 0;		// 1=OUTput,  0=INput 
	GpioCtrlRegs.GPAMUX2.bit.GPIO31 = 0;	// 0=GPIO,  
	GpioCtrlRegs.GPADIR.bit.GPIO31 = 0;		// 1=OUTput,  0=INput 

	// Configure gpio 34 as input for Busy signal from ADCs
	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;	// 0=GPIO, 
	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 0;		// 1=OUTput,  0=INput 

	// Configure gpios for manual conversion start
	GpioCtrlRegs.GPBMUX2.bit.GPIO56 = 0;	// 0=GPIO, 
	GpioCtrlRegs.GPBDIR.bit.GPIO56 = 1;		// 1=OUTput,  0=INput 
	GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 0;	// 0=GPIO,
	GpioCtrlRegs.GPBDIR.bit.GPIO57 = 1;		// 1=OUTput,  0=INput 

	// Configure pwm1 for EXTSOC
	GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1;		// 0=GPIO,  1=EPWM1A,  2=Resv,  3=Resv
	GpioCtrlRegs.GPADIR.bit.GPIO0 = 1;		// 1=OUTput,  0=INput 
	
	// Configure signals to use as ADC input
	GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 0;		// 0=GPIO, 
	GpioCtrlRegs.GPADIR.bit.GPIO1 = 1;		// 1=OUTput,  0=INput 
	GpioDataRegs.GPASET.bit.GPIO1 = 1;  	// Set high
	GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 0;		// 0=GPIO, 
	GpioCtrlRegs.GPADIR.bit.GPIO2 = 1;		// 1=OUTput,  0=INput 
	GpioDataRegs.GPASET.bit.GPIO2 = 1;  	// Set high
	
	
	// Manual Conversion Start pins
	GpioDataRegs.GPBSET.bit.GPIO56 = 1;		// Set high for manual CONVST triggering
	GpioDataRegs.GPBCLEAR.bit.GPIO57 = 1;	// Set low to allow ePWM triggered CONVST

	EDIS;

} // end GPIO_init

void XINTF_init()
{
	// Configures the XINTF module for controlling ADCs on Zone0 and Zone7
	// See the documentation in the doc folder for more information on timing
	
	EALLOW;

 	// Fast waitstates and the write buffer is set to buffer 3 writes.   
   	XintfRegs.XINTCNF2.bit.WRBUFF = 3; 			// Buffer up to 3 writes 
   
   	// XCLKOUT to SYSCLKOUT ratio. By default XCLKOUT = 1/8 SYSCLKOUT (200MHz) = (25MHz)
   	// XTIMCLK = SYSCLKOUT/2 = (100 MHz)
   	XintfRegs.XINTCNF2.bit.XTIMCLK = 1;
   	// XCLKOUT = XTIMCLK/2 = (50 MHz)
   	XintfRegs.XINTCNF2.bit.CLKMODE = 1;
   	// XCLKOUT = XTIMCLK/4 = (25 MHz)
   	XintfRegs.XINTCNF2.bit.BY4CLKMODE = 1;
   	// Enable XCLKOUT
   	XintfRegs.XINTCNF2.bit.CLKOFF = 0;  

#if (CE0)
	// Set up ADC1, if we have it enabled.
	// ADC1 is connected to XINTF Zone 0.

	// Zone read timing
    XintfRegs.XTIMING0.bit.XRDLEAD = 2;
    XintfRegs.XTIMING0.bit.XRDACTIVE = 6;
    XintfRegs.XTIMING0.bit.XRDTRAIL = 1; 

	// Zone write timing
	XintfRegs.XTIMING0.bit.XWRLEAD = 3;
    XintfRegs.XTIMING0.bit.XWRACTIVE = 1;
    XintfRegs.XTIMING0.bit.XWRTRAIL = 3;
	
	// Do not double all Zone read/write lead/active/trail timing 
    XintfRegs.XTIMING0.bit.X2TIMING = 0;
	
	// Zone will not sample READY 
    XintfRegs.XTIMING0.bit.USEREADY = 0;   
    XintfRegs.XTIMING0.bit.READYMODE = 0; 
	
	// To switch to 32-bit writes, make sure to also enable 32-bit XINTF using the XSIZE bits. 
    XintfRegs.XTIMING0.bit.XSIZE = 3;	// Size for x16 is 3 , Size for x32 is 1    
#endif

#if (CE7)
	// Set up ADC2, if we have it enabled.
	// ADC2 is connected to XINTF Zone 7.
	
	// Zone read timing
    XintfRegs.XTIMING7.bit.XRDLEAD = 2;   
    XintfRegs.XTIMING7.bit.XRDACTIVE = 6;  
    XintfRegs.XTIMING7.bit.XRDTRAIL = 1;  
	
	// Zone write timing
    XintfRegs.XTIMING7.bit.XWRLEAD = 3;
    XintfRegs.XTIMING7.bit.XWRACTIVE = 1;   
    XintfRegs.XTIMING7.bit.XWRTRAIL = 3;  
         
	// Do not double all Zone read/write lead/active/trail timing  
    XintfRegs.XTIMING7.bit.X2TIMING = 0;
       
	// Zone will not sample READY 
    XintfRegs.XTIMING7.bit.USEREADY = 0;   
    XintfRegs.XTIMING7.bit.READYMODE = 0; 
    
	// To switch to 32-bit writes, make sure to also enable 32-bit XINTF using the XSIZE bits. 
    XintfRegs.XTIMING7.bit.XSIZE = 3;	// Size for x16 is 3 , Size for x32 is 1    
#endif

	EDIS;
} // end xintf_init

void ADC_init()
{
	// Configures the ADS7865 ADCs on the controlCARD by writing commands to the ADCs. 
	// See ADS7865 datasheet for more information on ADC commands.
	
	Uint16 DAC_val;
#if (CE0)
	// Setup Vref on DAC 1
	address_ptr = ADC1;				// ADC1 is connected to Zone0, which is at address 0x4000
	*address_ptr = 0x105;			// RESET DEVICE
	*address_ptr = 0x103;			// Setup Vref DAC value read
	DAC_val = *ADC1;				// Read Vref DAC value
	*address_ptr = 0x101;			// Setup new Vref DAC value write
	*address_ptr = 0x2A3;			// Write new value to Vref DAC (value for 3.3V/2)
	*address_ptr = 0x103;			// Setup Vref DAC value read
	DAC_val = *ADC1;	    		// Read NEW Vref DAC value
	
	// ADC1
	// Setup SEQUENCER to convert pseudo-diff mode in the following sequence:
	// Convert CH(A/B)0+, on single CONVERT and BUSY for entire sequence (though only one in sequence anyway)
	address_ptr = ADC1;
	*address_ptr = 0x104;			// Setup new sequencer value write
	*address_ptr = 0xD00;			// Configure sequencer (S = 11: single convst and busy for sequence, SL = 01: sequence length 1)
									// Samples CH1 = 0: CHA0+/CHBo+, CM1 = 0:CHA0-/CHB0-; (pin A0)
	*address_ptr = 0x106;			// Setup sequencer value readback
	DAC_val = *ADC1;				// Read sequencer value
#endif
#if (CE7)
	// Setup Vref on DAC 2
	address_ptr = ADC2;				// ADC2 is connected to Zone7, which is at address 0x200000
	*address_ptr = 0x0105;			// RESET DEVICE
	*address_ptr = 0x0103;			// Setup Vref DAC value read
	DAC_val = *ADC2;				// Read Vref DAC value
	*address_ptr = 0x0101;			// Setup new Vref DAC value write
	*address_ptr = 0x02A3;			// Write new value to Vref DAC (value for 3.3V/2)
	*address_ptr = 0x0103;			// Setup Vref DAC value read
	DAC_val = *ADC2;				// Read NEW Vref DAC value
	
	// ADC2
	// Setup SEQUENCER to convert pseudo-diff mode in the following sequence:
	// individual convst and busy for each conversion for all channels in
	// the following order. 
	// CH(A/B)0+, CH(A/B)1+, then CH(A/B)1- 
	address_ptr = ADC2;
	*address_ptr = 0x104;			// Setup new sequencer value write
	*address_ptr = 0x324;			// Configure sequencer (S = 00: indiv. convst and busy for each conversion, SL = 11: sequence length 3)
									// Sequence is as follows: (CH1 = 0: CHA0+/CHB0+, CM1 = 0:CHA0-/CHB0-; (pin B0)
									//							CH2 = 1: CHA1+/CHB1+, CM2 = 0:CHA0-/CHB0-; (pin B1)
									//							CH3 = 0: CHA1-/CHB1-, CM3 = 1:CHA0-/CHB0-; (pin B2)
	*address_ptr = 0x106;			// Setup sequencer value readback
	DAC_val = *ADC2;	// Read sequencer value
#endif	
}

void EPWM_init()
{
	// Sets up the ePWM module for triggering CONVST for ADC2
	
	EALLOW;

	// Setup ePWM 1
	EPwm1Regs.TBCTL.bit.PRDLD = TB_IMMEDIATE;		// Set Immediate load
	EPwm1Regs.TBPRD = 0x0080;	 					// PWM frequency = 1 / period.
	EPwm1Regs.TBPHS.half.TBPHS = 0;
	EPwm1Regs.CMPA.half.CMPA = 0;					// Set duty 0% initially
	EPwm1Regs.CMPB = 0;								// Set duty 0% initially
	EPwm1Regs.TBCTR = 0;
	EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;
	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;
	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;

	EPwm1Regs.ETSEL.bit.SOCBEN = 0x1;   			// Enable extsoc1b event
	EPwm1Regs.ETSEL.bit.SOCBSEL = 0x2;  			// Set event to happen on TBCTR = TBPRD
	EPwm1Regs.ETPS.bit.SOCBPRD = 0x1;   			// Generate SoC on first event
	
	// Enable extsoc1B set polarity
	SysCtrlRegs.EXTSOCCFG.bit.EXTSOC1BEN = 0x1;     // Enable extsoc1b
	SysCtrlRegs.EXTSOCCFG.bit.EXTSOC1BPOLSEL= 0x1;  // Set inverted polarity (CONVST is active low)

	// Setup SOCADCCLK to run at 25Mhz
	SysCtrlRegs.HISPCP.bit.HSPCLK = 0x3;			// HSPCLK=SYSCLKOUT/8 (25Mhz)
	
	EDIS;
}

void Run_ADC()
{
	// Performs ADC conversions with both ADCs and stores results.

	Uint16 ADC1_res0,ADC1_res1;
	Uint16 ADC2_res0,ADC2_res1,ADC2_res2,ADC2_res3,ADC2_res4,ADC2_res6,DAC_val,junk;
	
	for(;;)
	{
	#if (CE0)
		// ADC 1 Loop; EZ Loop - started by GPIO, only two ADC values read 
		GpioDataRegs.GPBSET.bit.GPIO56 = 1;				// Make sure CONVST is high
		for (ndx=0; ndx<256; ndx++)
		{
			GpioDataRegs.GPBCLEAR.bit.GPIO56 = 1;		// Fire CONVST (active low)			
			while(GpioDataRegs.GPBDAT.bit.GPIO34==0){	// Wait for busy
				asm(" NOP");
		    }
			while(GpioDataRegs.GPBDAT.bit.GPIO34==1){	// Read after busy
				asm(" NOP");
		    }
			ADC1_res0 = *ADC1;    						// Read CHA0+ (A0)
			ADC1_res1 = *ADC1;    						// Read CHB0+ (A3)
			ADC1Data[ndx] = (ADC1_res0<<4);  			// Store result
			GpioDataRegs.GPBSET.bit.GPIO56 = 1;			// Release CONVST (return high)

		} // end 256 for loop
		ndx = 0;		
	#endif
	#if (CE7)
		// ADC 2 Loop
		
		// Align sequencer 
		address_ptr = ADC2;  				// Set ADC address
		*address_ptr = 0x106;			  	// Setup sequencer value readback
		DAC_val = (*ADC2 & 0x3);			// Read sequencer value

		while(DAC_val!= 0x1)			    // Loop until aligned
		{
			junk = *ADC2;    				// Dummy read to increment sequencer
			junk = *ADC2;    				// Dummy read to increment sequencer
			*address_ptr = 0x106;			// Setup sequencer value readback
			DAC_val = (*ADC2 & 0x3);		// Read sequencer value
		}

		for (ndx=0; ndx<256; ndx++)
		{
			asm(" NOP");   								// Good location for breakpoint

			// Convert 1st pair
			while(EPwm1Regs.ETFLG.bit.SOCB==0){  		// Wait for an event
				asm(" NOP");  							// No event, do nothing
			}				
			EPwm1Regs.ETCLR.bit.SOCB = 1;  				// Clear flag
			
			while(GpioDataRegs.GPBDAT.bit.GPIO34==0){	// Wait for busy
				asm(" NOP");
		    }
			while(GpioDataRegs.GPBDAT.bit.GPIO34==1){	// Read after busy
				asm(" NOP");
		    }
			ADC2_res0 = *ADC2;    						// Read CHA0+ (B0)
			ADC2_res3 = *ADC2;							// Read CHB0+ (B3)
		
			// convert 2nd pair
			while(EPwm1Regs.ETFLG.bit.SOCB==0){			// Wait for an event
				asm(" NOP");  							// No event, do nothing
			}
			EPwm1Regs.ETCLR.bit.SOCB = 1;  				// Clear flag

			while(GpioDataRegs.GPBDAT.bit.GPIO34==0){	// Wait for busy
				asm(" NOP");
		    }
			while(GpioDataRegs.GPBDAT.bit.GPIO34==1){	// Read after busy
				asm(" NOP");
		    }
			ADC2_res1 = *ADC2;    						// Read CHA1+ (B1)
			ADC2_res4 = *ADC2;    						// Read CHB1+ (B4)	
			
			// convert 3rd pair
			while(EPwm1Regs.ETFLG.bit.SOCB==0){			// Wait for an event
				asm(" NOP");  							// No event, do nothing
			}
			EPwm1Regs.ETCLR.bit.SOCB = 1;  				// Clear flag
			while(GpioDataRegs.GPBDAT.bit.GPIO34==0){	// Wait for busy
				asm(" NOP");
		    }
			while(GpioDataRegs.GPBDAT.bit.GPIO34==1){	// Read after busy
				asm(" NOP");
		    }
			ADC2_res2 = *ADC2;							// Read CHA1- (B2)
			ADC2_res6 = *ADC2;							// Read CHB1- (B6)	

			ADC2Data[ndx] = (ADC2_res0<<4);				// Store result
					
		} // end 256 for loop
		ndx = 0;
	#endif		
	} // end for loop
}// end ADC Run


//===========================================================================
// No more.
//===========================================================================
